// =====================================================================================================================
//  fennec, a free and open source game engine
//  Copyright (C) 2025  Medusa Slockbower
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <https://www.gnu.org/licenses/>.
// =====================================================================================================================

///
/// \file relational.h
/// \brief \ref page_fennec_math_relational
///
///
/// \details
/// \author Medusa Slockbower
///
/// \copyright Copyright © 2025 Medusa Slockbower ([GPLv3](https://www.gnu.org/licenses/gpl-3.0.en.html))
///
///

#ifndef FENNEC_MATH_RELATIONAL_H
#define FENNEC_MATH_RELATIONAL_H
#include <fennec/lang/types.h>

// TODO: Document

namespace fennec
{

template<typename genType, typename genBType = bool_t>
constexpr genBType lessThan(genType x, genType y)
	{ return x < y; }

template<typename genType, typename genBType = bool_t, size_t...i>
constexpr vector<genBType, i...> lessThan(const vector<genType, i...>& x, const vector<genType, i...>& y)
	{ return vector<genBType, i...>(x[i] < y[i]...); }

template<typename genType, typename genBType = bool_t>
constexpr genBType lessThanEqual(genType x, genType y)
	{ return x <= y; }

template<typename genType, typename genBType = bool_t, size_t...i>
constexpr vector<genBType, i...> lessThanEqual(const vector<genType, i...>& x, const vector<genType, i...>& y)
	{ return vector<genBType, i...>(x[i] <= y[i]...); }

template<typename genType, typename genBType = bool_t>
constexpr genBType greaterThan(genType x, genType y)
	{ return x > y; }

template<typename genType, typename genBType = bool_t, size_t...i>
constexpr vector<genBType, i...> greaterThan(const vector<genType, i...>& x, const vector<genType, i...>& y)
	{ return vector<genBType, i...>(x[i] > y[i]...); }

template<typename genType, typename genBType = bool_t>
constexpr genBType greaterThanEqual(genType x, genType y)
	{ return x >= y; }

template<typename genType, typename genBType = bool_t, size_t...i>
constexpr vector<genBType, i...> greaterThanEqual(const vector<genType, i...>& x, const vector<genType, i...>& y)
	{ return vector<genBType, i...>(x[i] >= y[i]...); }

template<typename genType, typename genBType = bool_t>
constexpr genBType equal(genType x, genType y)
	{ return x == y; }

template<typename genType, typename genBType = bool_t, size_t...i>
constexpr vector<genBType, i...> equal(const vector<genType, i...>& x, const vector<genType, i...>& y)
	{ return vector<genBType, i...>(x[i] == y[i]...); }

template<typename genType, typename genBType = bool_t>
constexpr genBType notEqual(genType x, genType y)
	{ return x != y; }

template<typename genType, typename genBType = bool_t, size_t...i>
constexpr vector<genBType, i...> notEqual(const vector<genType, i...>& x, const vector<genType, i...>& y)
	{ return vector<genBType, i...>(x[i] != y[i]...); }

template<typename genBType = bool_t, size_t...i>
constexpr genBType any(const vector<genBType, i...>& x)
	{ return (x[i] || ...); }

template<typename genBType = bool_t, size_t...i>
constexpr genBType all(const vector<genBType, i...>& x)
	{ return (x[i] && ...); }

}



#endif // FENNEC_MATH_RELATIONAL_H
